#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stdarg.h>
#include "Matrix.h"

#define EPS 1e-10

Matrix* MatrixNew (int n, int m)
{
	Matrix* A;	
	int i;
	A = (Matrix*)malloc(sizeof(Matrix));
	A->M = (double**)malloc(n*sizeof(double*));
	for (i=0; i<n; i++)
		(A->M)[i] = (double*)malloc(m*sizeof(double));
	A->n = n;
	A->m = m;
	return A;
}

void MatrixDelete (Matrix* M)
{
	int i;
	for (i=0; i < M->n; i++)
		free ((M->M)[i]);	
	free (M->M);
	free (M);
}

Matrix* MatrixCopy (Matrix* A)
{
	Matrix* M;
	int i, j;

	M = MatrixNew (A->n, A->m);

	for (i=0; i < A->n; i++)
		for (j=0; j < A->m; j++)
			(M->M)[i][j] = (A->M)[i][j];
	return M;
}

Matrix* MatrixE (int n)
{
	Matrix* E;
	int i, j;

	E = MatrixNew (n, n);

	for (i=0; i<n; i++)
		for (j=0; j<n; j++) {
			if (i == j)
				(E->M)[i][j] = 1;
			else
				(E->M)[i][j] = 0;
		}
	return E;
}

void MatrixSetRow (Matrix* A, int i, ...)
{
	int j;
	va_list ap;

	va_start (ap, i);
	for (j=0; j < (A->m); j++)
		(A->M)[i][j] = va_arg (ap, double);

	va_end (ap);
}

void MatrixSetColumn (Matrix* A, int j, ...)
{
	int i;
	va_list ap;

	va_start (ap, j);
	for (i=0; i < (A->n); i++)
		(A->M)[i][j] = va_arg (ap, double);

	va_end (ap);
}

void MatrixSetElement (Matrix* A, int i, int j, double a)
{

	(A->M)[i][j] = a;
}

void MatrixGetRow (Matrix* A, int i, double* a)
{
	int j;
	for (j=0; j<A->m; j++)
		a[j] = (A->M)[i][j];

}

void MatrixGetColumn (Matrix* A, int j, double* a)
{
	int i;
	for (i=0; i<A->n; i++)
		a[i] = (A->M)[i][j];

}

double MatrixElement (Matrix* A, int i, int j)
{
	return (A->M)[i][j];
}

Matrix* MatrixMult (Matrix* A, Matrix* B)
{
	int i, j, k;
	Matrix* X;

	if (A->m != B->n)
		return NULL;

	X = MatrixNew (A->n, B->m);

	for (i=0; i< A->n; i++)
		for (j=0; j< B->m; j++) {
			(X->M)[i][j] = 0.0;
			for (k = 0; k < A->m; k++)
				(X->M)[i][j] += (A->M)[i][k]*(B->M)[k][j];
		}

	return X;
}

Matrix* MatrixTranspose (Matrix* A)
{
	int i,j;
	Matrix* X;

	X = MatrixNew (A->m, A->n);

	for (i=0; i < A->m; i++)
		for (j=0; j < A->n; j++)
				(X->M)[i][j] = (A->M)[j][i];

	return X;
}

static void MatrixSwapRows (Matrix* A, int i, int j)
{
	double* tmp;
	tmp = (A->M)[i];
	(A->M)[i] = (A->M)[j];
	(A->M)[j] = tmp;
}

static void MatrixSwapColumns (Matrix* A, int i, int j)
{
	int k;
	double tmp;

	for (k = 0; k< A->n; k++) {
		tmp = (A->M)[k][i];
		(A->M)[k][i] = (A->M)[k][j];
		(A->M)[k][j] = tmp;
	}
}

void MatrixPrint (Matrix* A) 
{
	int i, j;

	for (i=0; i< (A->n); i++) {
		for (j=0; j< (A->m); j++)
			printf ("%f ", (A->M)[i][j]);
		printf ("\n");
	}
}

static void Swap (int r[], int i, int j)
{
	int tmp;
	tmp = r[i];
	r[i] = r[j];
	r[j] = tmp;
}

static Matrix* Reorder (Matrix* A, int r[])
{
	int i, j;
	Matrix* X;
	X = MatrixNew (A->n, A->m);

	for (i = 0; i < A->n; i++)
		for (j = 0; j < A->m; j++)
			(X->M)[r[i]][j] = (A->M)[i][j];
	
	return X;
}

static double RowMaxElement (Matrix* A, int i, int* j)
{
	int k;
	double a, x = 0;

	for (k = 0; k < A->n; k++) {
		a = fabs((A->M)[i][k]);
		if (a > x) {
			x = a;
			*j = k;
		}				
	}

	return x;
}

static double  MatrixMaxElement (Matrix* A, int i, int j, int* k, int* l)
{
	double a, x = 0;
	int r, s;
	*k = 0; *l = 0;

	for (r = i; r < A->n; r++) 
		for (s = j; s < A->m; s++) {
			a = fabs((A->M)[r][s]);
			if (a > x) {
				x  = a;
				*k = r;
				*l = s;
			}
		}	
	return x;
}

static int GaussElimination (Matrix* A, double* det)
{
	int i, j, k, r, s;	
	double a, D = 1;
	int rank = 0;

	for (i = 0; i < A->n; i++) {
		if (MatrixMaxElement (A, i, i, &r, &s) < EPS)
			return rank;
		rank++;
		
		MatrixSwapRows (A, i, r);
		MatrixSwapColumns (A, i, s);

		a = (A->M)[i][i];
		D *= a*pow (-1, 2*i - r - s);

		for (j=i; j < A->m; j++)
			(A->M)[i][j] /= a;

		for (k=i+1; k < A->n; k++) {
			a = (A->M)[k][i];
			for (j=i; j < A->m; j++)
				(A->M)[k][j] -= (A->M)[i][j]*a;

		}
	}
	if (det != NULL)
		*det = D;

	return rank;
}

int MatrixRank (Matrix* A)
{
	int rank;
	Matrix* _A; 
	_A = MatrixCopy (A);
	rank = GaussElimination (_A, NULL);
	MatrixDelete (_A);
	return rank;
}

double MatrixDeterminant (Matrix* A)
{
	double D;
	Matrix* _A; 
	_A = MatrixCopy (A);
	GaussElimination (_A, &D);
	MatrixDelete (_A);
	return D;
}

static Matrix* GaussJordanElimination (Matrix* A, Matrix* B)
{
	int i, j, k, r, s;	
	double a;
	int R[A->n];

	for (i=0; i < A->n; i++)
		R[i] = i;
/* Forward */

	for (i = 0; i < A->n ; i++) {

		if (RowMaxElement (A, i, &r) < EPS) {
			if (RowMaxElement (B, i, &s) < EPS)
				continue;
			else
				return NULL;
		}
		
		MatrixSwapColumns (A, i, r);
		Swap (R, i, r);

		a = (A->M)[i][i];
		for (j=i; j < A->m; j++)
			(A->M)[i][j] /= a;

		for (j=0; j < B->m; j++)
			(B->M)[i][j] /= a;

		for (k=i+1; k < A->n; k++) {
			a = (A->M)[k][i];
			for (j=i; j < A->m; j++)
				(A->M)[k][j] -= (A->M)[i][j]*a;			
			for (j=0; j < B->m; j++)
				(B->M)[k][j] -= (B->M)[i][j]*a;
		}
	}

/* Backward */

	for (j = A->n - 1; j>0; j--) {
		for (i=j-1; i>=0; i--) {
			a = (A->M)[i][j];
			(A->M)[i][j] = 0;
			for (k= 0; k < B->m; k++)
				(B->M)[i][k] -= (B->M)[j][k]*a;
		}
	}

	return Reorder (B, R);
}

Matrix* MatrixSolve (Matrix* A, Matrix* B)
{
	Matrix *_A, *_B, *X;
	if (A->n != B->n)
		return NULL;

	_A = MatrixCopy (A);
	_B = MatrixCopy (B);
	X = GaussJordanElimination (_A, _B);
	MatrixDelete (_A);
	MatrixDelete (_B);
	return X;
}

Matrix* MatrixInverse (Matrix* A)
{
	int n;
	Matrix* E;
	Matrix* X;
	Matrix* _A;

	if (A->n != A->m)
		return NULL;

	n = A->n;

	E = MatrixE (n);
	_A = MatrixCopy (A);

	X = GaussJordanElimination (_A,E);
	MatrixDelete (E);
	MatrixDelete (_A);

	return X;
}

/*
  Local Variables:
  compile-command:"gcc -Wall -g -c Matrix.c -lm"
  End:
*/
